VERSION 5.00
Begin VB.UserControl Histogram 
   Appearance      =   0  'Flat
   ClientHeight    =   2760
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   3000
   ScaleHeight     =   2760
   ScaleWidth      =   3000
   ToolboxBitmap   =   "Histgram.ctx":0000
   Begin HistogramProject.Gauge Bar 
      Height          =   2175
      Index           =   0
      Left            =   120
      TabIndex        =   1
      Top             =   120
      Visible         =   0   'False
      Width           =   255
      _ExtentX        =   450
      _ExtentY        =   3836
      Orientation     =   2
      Style           =   3
   End
   Begin VB.HScrollBar ScrollBar 
      Height          =   255
      Left            =   120
      TabIndex        =   0
      Top             =   2400
      Width           =   2775
   End
End
Attribute VB_Name = "Histogram"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' *********************************************
' A histogram.
' *********************************************
Option Explicit

' Defer arranging controls and displaying data.
Private loading As Boolean

' Gauge styles.
Public Enum HistStyle
    styleBar
    styleDial
    stylePic
    styleTic
    styleWid
End Enum
Private the_style As HistStyle

' The data values.
Private the_value() As Long

' The number of bars.
Private num_bars As Integer

' The number of data values.
Private num_values As Integer

' Default property values.
Private Const dflt_BackColor = &H8000000F   ' Light gray.
Private Const dflt_ForeColor = vbBlack
Private Const dflt_Style = styleTic
Private Const dflt_Enabled = True
Private Const dflt_TicWidth = 4
Private Const dflt_Max = 100
Private Const dflt_Min = 0
Private Const dflt_NumBars = 1
Private Const dflt_NumValues = 1
Private Const dflt_Value = 0

' ************
' Events
' ************
' The user has changed the Index-th data value.
Public Event ValueChanged(Index As Integer)
' *********************************************
' Arrange the controls.
' *********************************************
Private Sub ArrangeControls()
Const GAP = 60

Dim wid As Single
Dim hgt As Single
Dim tp As Single
Dim i As Integer
Dim X As Single

    If loading Then Exit Sub
    
    ' See how wide the control needs to be.
    wid = NumBars * (Bar(0).Width + GAP) - GAP
    Width = wid + (Width - ScaleWidth)
    
    ' See if we will need the scroll bar.
    If NumBars >= NumValues Then
        ' We don't need the scroll bar.
        ' Make the bars as tall as possible.
        hgt = ScaleHeight
    Else
        ' We do need the scroll bar.
        ' Place the scroll bar.
        tp = ScaleHeight - ScrollBar.Height
        ScrollBar.Move 0, tp, ScaleWidth
    
        ' Leave room for the scroll bar.
        hgt = ScrollBar.Top - GAP
    End If
    
    ' Arrange the gauges.
    wid = Bar(0).Width
    X = 0
    For i = 1 To NumBars
        Bar(i).Move X, 0, wid, hgt
        X = X + wid + GAP
    Next i
    
    ' Make the bars visible.
    For i = 1 To NumBars
        Bar(i).Visible = True
    Next i

    ' Make the scroll bar visible or not as appropriate.
    If NumBars >= NumValues Then
        ScrollBar.Visible = False
    Else
        ScrollBar.Visible = True
    
        ' Set the scroll bar's properties.
        ScrollBar.Max = NumValues - NumBars
        ScrollBar.LargeChange = NumBars
    End If
End Sub
' *********************************************
' Display the values.
' *********************************************
Private Sub ShowValues()
Dim off As Integer
Dim i As Integer

    off = ScrollBar.Value
    For i = 1 To NumBars
        Bar(i).Value = Value(i + off)
    Next i
End Sub

' *********************************************
' Get the stripe width.
' *********************************************
Public Property Get TicWidth() As Long
Attribute TicWidth.VB_Description = "Size of tic marks in pixels for styleTic"
    TicWidth = Bar(0).TicWidth
End Property
' *********************************************
' Set the tic width for the bar gauges.
' *********************************************
Public Property Let TicWidth(val As Long)
Dim i As Integer
    
    For i = 0 To NumBars
        Bar(i).TicWidth = val
    Next i

    PropertyChanged "TicWidth" ' Tell VB.
End Property
' *********************************************
' Get the bars' background color.
' *********************************************
Public Property Get BarBackColor() As OLE_COLOR
    BarBackColor = Bar(0).BackColor
End Property
' *********************************************
' Set the bars' background color.
' *********************************************
Public Property Let BarBackColor(clr As OLE_COLOR)
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).BackColor = clr
    Next i

    PropertyChanged "BarBackColor" ' Tell VB.
End Property
' *********************************************
' Get the background color.
' *********************************************
Public Property Get BackColor() As OLE_COLOR
Attribute BackColor.VB_Description = "For styles other than Dial, fills the gauge when Value = Min"
Attribute BackColor.VB_UserMemId = -501
    BackColor = UserControl.BackColor
End Property
' *********************************************
' Set the background color.
' *********************************************
Public Property Let BackColor(clr As OLE_COLOR)
    UserControl.BackColor = clr
    PropertyChanged "BackColor" ' Tell VB.
End Property
' *********************************************
' Set the tic color.
' *********************************************
Public Property Let BarForeColor(clr As OLE_COLOR)
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).ForeColor = clr
    Next i

    PropertyChanged "BarForeColor" ' Tell VB.
End Property
' *********************************************
' Get the tic color.
' *********************************************
Public Property Get BarForeColor() As OLE_COLOR
Attribute BarForeColor.VB_Description = "For styles other than styleDial, fills the gauge when Value = Max"
Attribute BarForeColor.VB_UserMemId = -513
    BarForeColor = Bar(0).ForeColor
End Property
' *********************************************
' Set the minimum value.
' *********************************************
Public Property Let Min(val As Long)
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).Min = val
    Next i

    PropertyChanged "Min" ' Tell VB.
End Property
' *********************************************
' Set the number of bars.
' *********************************************
Public Property Let NumBars(val As Integer)
Dim i As Integer
    
    If val < 1 Then Exit Property
    
    ' Unload bars no longer needed.
    For i = val + 1 To num_bars
        Unload Bar(i)
    Next i
    
    ' Load bars now needed.
    For i = num_bars + 1 To val
        Load Bar(i)
    Next i
    
    ' Save the new value.
    num_bars = val
    
    ' Arrange the controls.
    ArrangeControls
    
    PropertyChanged "NumBars" ' Tell VB.
End Property
' *********************************************
' Set the number of values.
' *********************************************
Public Property Let NumValues(val As Integer)
    If val < 1 Then Exit Property
    
    ' Redimension the value array.
    ReDim Preserve the_value(1 To val)
    
    ' Save the number of values.
    num_values = val
    
    ' Arrange the controls.
    ArrangeControls
    
    PropertyChanged "Value"     ' Tell VB.
    PropertyChanged "NumValues" ' Tell VB.
End Property
' *********************************************
' Set the values.
' *********************************************
Public Property Let Value(i As Integer, val As Long)
Attribute Value.VB_Description = "The value selected"
    If i < 1 Or i > NumValues Then Exit Property

    the_value(i) = val
    
    ' Redisplay the values (we don't need to
    ' rearrange the controls).
    ShowValues
    
    PropertyChanged "Value" ' Tell VB.
End Property
' *********************************************
' Get the minimum value.
' *********************************************
Public Property Get Min() As Long
    Min = Bar(0).Min
End Property
' *********************************************
' Get the number of bars.
' *********************************************
Public Property Get NumBars() As Integer
    NumBars = num_bars
End Property
' *********************************************
' Get the number of values.
' *********************************************
Public Property Get NumValues() As Integer
    NumValues = num_values
End Property
' *********************************************
' See if the bars are enabled.Get the number of values.
' *********************************************
Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Determines whether the user can modify the bar values"
Attribute Enabled.VB_UserMemId = -514
    Enabled = Bar(0).Enabled
End Property

' *********************************************
' Set the maximum value.
' *********************************************
Public Property Let Max(val As Long)
Attribute Max.VB_Description = "Maximum allowed Value"
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).Max = val
    Next i

    PropertyChanged "Max" ' Tell VB.
End Property
' *********************************************
' Set Enabled for the bars.
' *********************************************
Public Property Let Enabled(val As Boolean)
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).Enabled = val
    Next i

    PropertyChanged "Enabled" ' Tell VB.
End Property

' *********************************************
' Get the maximum value.
' *********************************************
Public Property Get Max() As Long
    Max = Bar(0).Max
End Property
' *********************************************
' Set the style.
' *********************************************
Public Property Let Style(val As HistStyle)
Dim i As Integer

    For i = 0 To NumBars
        Bar(i).Style = val
    Next i

    PropertyChanged "Style" ' Tell VB.
End Property
' *********************************************
' Get the style.
' *********************************************
Public Property Get Style() As HistStyle
Attribute Style.VB_Description = "Style of gauge (styleBar, stylePic, etc.)"
    Style = Bar(0).Style
End Property
' *********************************************
' Return the value.
' *********************************************
Public Property Get Value(i As Integer) As Long
    If i < 1 Or i > NumValues Then
        Value = 0
    Else
        Value = the_value(i)
    End If
End Property

' *********************************************
' The user has modified a value. Save the new
' value.
' *********************************************
Private Sub Bar_Change(Index As Integer)
Dim i As Integer

    ' Calculate the index of the changed value.
    i = ScrollBar.Value + Index

    the_value(i) = Bar(Index).Value

    ' Raise the value changed event.
    RaiseEvent ValueChanged(i)
    
    PropertyChanged "Value" ' Tell VB.
End Sub

' ************************************************
' Redisplay the values.
' ************************************************
Private Sub ScrollBar_Change()
    ShowValues
End Sub
' ************************************************
' Redisplay the values.
' ************************************************
Private Sub ScrollBar_Scroll()
    ShowValues
End Sub

' *********************************************
' Set default property values.
' *********************************************
Private Sub UserControl_InitProperties()
Dim i As Integer

    loading = True
    
    BackColor = Ambient.BackColor
    BarBackColor = BackColor
    BarForeColor = Ambient.ForeColor
    Max = dflt_Max
    Min = dflt_Min
    NumBars = dflt_NumBars
    NumValues = dflt_NumValues
    Style = dflt_Style
    Enabled = dflt_Enabled
    TicWidth = dflt_TicWidth
    
    For i = 1 To NumValues
        Value(i) = dflt_Value
    Next i

    loading = False
    ArrangeControls
End Sub
' *********************************************
' Read the property values.
' *********************************************
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
Dim i As Integer
    
    loading = False
    
    BackColor = PropBag.ReadProperty("BackColor", Ambient.BackColor)
    BarBackColor = PropBag.ReadProperty("BarBackColor", Ambient.BackColor)
    BarForeColor = PropBag.ReadProperty("BarForeColor", Ambient.ForeColor)
    Max = PropBag.ReadProperty("Max", dflt_Max)
    Min = PropBag.ReadProperty("Min", dflt_Min)
    NumBars = PropBag.ReadProperty("NumBars", dflt_NumBars)
    NumValues = PropBag.ReadProperty("NumValues", dflt_NumValues)
    Style = PropBag.ReadProperty("Style", dflt_Style)
    Enabled = PropBag.ReadProperty("Enabled", dflt_Enabled)
    TicWidth = PropBag.ReadProperty("TicWidth", dflt_TicWidth)

    For i = 1 To NumValues
        the_value(i) = PropBag.ReadProperty( _
            "Value" & Format$(i), dflt_Value)
    Next i
    
    loading = False
    ArrangeControls
End Sub
' *********************************************
' Save the property values.
' *********************************************
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
Dim i As Integer

    PropBag.WriteProperty "BackColor", BackColor, Ambient.BackColor
    PropBag.WriteProperty "BarBackColor", BarBackColor, Ambient.BackColor
    PropBag.WriteProperty "BarForeColor", BarForeColor, Ambient.ForeColor
    PropBag.WriteProperty "Max", Max, dflt_Max
    PropBag.WriteProperty "Min", Min, dflt_Min
    PropBag.WriteProperty "NumBars", NumBars, dflt_NumBars
    PropBag.WriteProperty "NumValues", NumValues, dflt_NumValues
    PropBag.WriteProperty "Style", Style, dflt_Style
    PropBag.WriteProperty "Enabled", Enabled, dflt_Enabled
    PropBag.WriteProperty "TicWidth", TicWidth, dflt_TicWidth

    For i = 1 To NumValues
        PropBag.WriteProperty "Value" & Format$(i), _
            Value(i), dflt_Value
    Next i
End Sub
